@@ -1,3 +1,4 @@
+import logging
 import numpy as np
 import pandas as pd
 from pathlib import Path
@@ -8,7 +9,8 @@ from astropy.io import fits
 from astropy.wcs import WCS
 
 from .model import EmissionFitting
-from .tools import label_decomposition, LineFinder, UNITS_LATEX_DICT, latex_science_float
+from .tools import label_decomposition, LineFinder, UNITS_LATEX_DICT, latex_science_float, DISPERSION_UNITS,\
+                   FLUX_DENSITY_UNITS, unit_convertor
 from .plots import LiMePlots, STANDARD_PLOT, STANDARD_AXES, colorDict, save_close_fig_swicth
 from .io import _LOG_DTYPES_REC, _LOG_EXPORT, _LOG_COLUMNS, load_lines_log, save_line_log
 from .model import gaussian_profiles_computation, linear_continuum_computation
@@ -16,7 +18,8 @@ from .model import gaussian_profiles_computation, linear_continuum_computation
 from matplotlib import pyplot as plt, colors, cm, gridspec, rc_context
 from matplotlib.widgets import SpanSelector
 from matplotlib.widgets import RadioButtons
-from lime import _logger
+
+_logger = logging.getLogger('LiMe')
 
 try:
     import mplcursors
@@ -76,7 +79,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
     """
 
     def __init__(self, input_wave=None, input_flux=None, input_err=None, redshift=0, norm_flux=1.0, crop_waves=None,
-                 inst_FWHM = np.nan, units_wave='A', units_flux='erg/cm^2/s/A'):
+                 inst_FWHM = np.nan, units_wave='A', units_flux='Flam', masked_pixels=None):
 
         # Load parent classes
         LineFinder.__init__(self)
@@ -94,6 +97,7 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         self.inst_FWHM = inst_FWHM
         self.units_wave = units_wave
         self.units_flux = units_flux
+        self._masked_inputs = False
 
         # Checks spectra units
         for arg in ['units_wave', 'units_flux']:
@@ -105,15 +109,17 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         # Check if spectrum redshift and flux normalization flux are provided
         for arg in ['norm_flux', 'redshift']:
             if self.__getattribute__(arg) is None:
-                _logger.note(f'No value provided for the {arg}')
+                _logger.debug(f'No value provided for the {arg}')
 
         # Start cropping the input spectrum if necessary
         if crop_waves is not None:
-            idcs_cropping = (input_wave >= crop_waves[0]) & (input_wave <= crop_waves[1])
-            input_wave = input_wave[idcs_cropping]
-            input_flux = input_flux[idcs_cropping]
+            idcs_crop = np.searchsorted(input_wave, crop_waves)
+            input_wave = input_wave[idcs_crop[0]:idcs_crop[1]]
+            input_flux = input_flux[idcs_crop[0]:idcs_crop[1]]
             if input_err is not None:
-                input_err = input_err[idcs_cropping]
+                input_err = input_err[idcs_crop[0]:idcs_crop[1]]
+            if masked_pixels is not None:
+                masked_pixels = masked_pixels[idcs_crop[0]:idcs_crop[1]]
 
         # Apply the redshift correction
         if input_wave is not None:
@@ -140,11 +146,31 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
                     _logger.warning(f'Input spectrum {arg} array has {np.sum(nan_entries)} this could bring issues in your fittings'
                                     f'formats: {list(UNITS_LATEX_DICT.keys())}')
                 if dimensions != 1:
-                    _logger.bug(f'Input spectrum {arg} array has a number of dimensions equal to {dimensions}. Fitting issues'
+                    _logger.warning(f'Input spectrum {arg} array has a number of dimensions equal to {dimensions}. Fitting issues'
                                 f' will stop the script running')
             else:
                 if arg in ['wave', 'flux']:
-                    _logger.warning(f'No {arg} array introduced. This will bring issues in your fittings.')
+                    _logger.warning(f'No {arg} array introduced. This may bring issues in your fittings.')
+
+        # Masked the arrays if requested
+        if masked_pixels is not None:
+            self.wave = np.ma.masked_array(self.wave, masked_pixels)
+            self.wave_rest = np.ma.masked_array(self.wave_rest, masked_pixels)
+            self.flux = np.ma.masked_array(self.flux, masked_pixels)
+
+            if self.err_flux is not None:
+                self.err_flux = np.ma.masked_array(self.err_flux, masked_pixels)
+
+        # Check for masked arrays
+        check_mask = np.zeros(3).astype(bool)
+        for i, arg in enumerate(['wave', 'wave_rest', 'flux']):
+            if np.ma.is_masked(self.__getattribute__(arg)):
+                check_mask[i] = True
+
+        if np.any(check_mask):
+            self._masked_inputs = True
+            if ~np.all(check_mask):
+                _logger.warning(f'Make sure *both* your input wavelength and flux array are masked arrays')
 
         # Generate empty dataframe to store measurement use cwd as default storing folder
         self.log = pd.DataFrame(np.empty(0, dtype=_LOG_DTYPES_REC))
@@ -225,15 +251,25 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
         self._decimal_wave = True if '.' in self.line else False
 
         # Get spectrum line and continua band indeces
-        idcsEmis, idcsCont = self.define_masks(self.wave, self.mask*(1 + self.redshift), line_mask_entry=fit_conf.get(f'{self.line}_mask'))
+        idcsEmis, idcsCont = self.define_masks(self.wave, self.mask*(1 + self.redshift),
+                                               line_mask_entry=fit_conf.get(f'{self.line}_mask'))
 
         # Integrated line properties
         emisWave, emisFlux = self.wave[idcsEmis], self.flux[idcsEmis]
         contWave, contFlux = self.wave[idcsCont], self.flux[idcsCont]
         err_array = self.err_flux[idcsEmis] if self.err_flux is not None else None
 
+        # Review the transition bands before
+        emis_band_lengh = emisWave.size if not np.ma.is_masked(emisWave) else np.sum(~emisWave.mask)
+        cont_band_length = contWave.size if not np.ma.is_masked(contWave) else np.sum(~contWave.mask)
+        if emis_band_lengh/emisWave.size < 0.5:
+            _logger.warning(f'The line band for {self.line} has very few valid pixels')
+
+        if cont_band_length/contWave.size < 0.5:
+            _logger.warning(f'The continuum band for {self.line} has very few valid pixels')
+
         # Store error very small mask
-        if emisWave.size <= 1:
+        if emis_band_lengh <= 1:
             if self.observations == 'no':
                 self.observations = 'Small_line_band'
             else:
@@ -449,6 +485,44 @@ class Spectrum(EmissionFitting, LiMePlots, LineFinder):
 
         return
 
+    def convert_units(self, units_wave=None, units_flux=None, norm_flux=None):
+
+        if units_wave is not None:
+
+            if units_wave in DISPERSION_UNITS:
+                self.wave = unit_convertor(self.units_wave, units_wave, wave_array=self.wave)
+                self.wave_rest = self.wave/(1+self.redshift)
+            else:
+                _logger.warning(f'- Dispersion units {units_wave} not recognized for conversion. '
+                                f'Please use {DISPERSION_UNITS} to convert from {self.units_wave}')
+
+            # Reflect the new units
+            self.units_wave = units_wave
+
+
+        if units_flux is not None:
+
+            if units_flux in FLUX_DENSITY_UNITS:
+                self.flux = unit_convertor(self.units_flux, units_flux, wave_array=self.wave, flux_array=self.flux,
+                                           dispersion_units=self.units_wave)
+            else:
+                _logger.warning(f'- Dispersion units {units_flux} not recognized for conversion. '
+                                f'Please use {FLUX_DENSITY_UNITS} to convert from {self.units_flux}')
+
+            # Reflect the new units
+            self.units_flux = units_flux
+
+        if norm_flux is not None:
+
+            # Undo previous normalization
+            self.flux = self.flux * self.norm_flux
+
+            # New normalization
+            self.norm_flux = norm_flux
+            self.flux = self.flux/self.norm_flux
+
+        return
+
     def clear_fit(self):
         super().__init__()
         return
@@ -844,7 +918,7 @@ class CubeInspector(Spectrum):
 
     def __init__(self, wave, cube_flux, image_bg, image_fg=None, contour_levels=None, color_norm=None,
                  redshift=0, lines_log_address=None, fits_header=None, plt_cfg={}, ax_cfg={},
-                 ext_suffix='_LINESLOG', mask_file=None, units_wave='A', units_flux='erg/cm^2/s/A'):
+                 ext_suffix='_LINESLOG', mask_file=None, units_wave='A', units_flux='Flam'):
 
         """
         This class provides an interactive plot for IFU (Integra Field Units) data cubes consisting in two figures:
@@ -910,7 +984,7 @@ class CubeInspector(Spectrum):
         #TODO add frame argument
 
         # Assign attributes to the parent class
-        super().__init__(wave, input_flux=None, redshift=redshift, norm_flux=1, units_wave=units_wave, units_flux=units_flux)
+        super().__init__(input_wave=np.zeros(1), input_flux=np.zeros(1), redshift=redshift, norm_flux=1, units_wave=units_wave, units_flux=units_flux)
 
         # Data attributes
         self.grid_mesh = None
@@ -958,8 +1032,6 @@ class CubeInspector(Spectrum):
 
         # Adjust the default theme
         self.fig_conf['figure.figsize'] = (18, 6)
-        # self.fig_conf['ytick.labelsize'] = 10
-        # self.fig_conf['xtick.labelsize'] = 10
 
         # Update to the user configuration
         self.fig_conf = {**self.fig_conf, **plt_cfg}
